package com.csc.library.zeesecurity.datasource.cassandra.manager;

import java.util.Date;
import java.util.Optional;
import java.util.function.Function;

import javax.ejb.Singleton;

import com.csc.library.cassandra.Cassandra3Manager;
import com.csc.library.cassandra.annotation.Column;
import com.csc.library.cassandra.util.PatternSqlBase;
import com.csc.library.status.StatusMessage;
import com.csc.library.system.layer.DatasourceEnvironment;
import com.csc.library.system.layer.DatasourceLayer;
import com.csc.library.system.layer.DatasourceType;
import com.csc.library.utilities.CSCCheckUtility;
import com.csc.library.utilities.MyLog;
import com.csc.library.zeesecurity.databean.entity.EmployeeTimeConfig;
import com.csc.library.zeesecurity.databean.entity.impl.EmployeeTimeConfigAbstract;
import com.csc.library.zeesecurity.databean.manager.EmployeeTimeConfigManager;
import com.datastax.driver.core.ResultSet;
import com.datastax.driver.core.Row;
import com.datastax.driver.mapping.annotations.Accessor;
import com.datastax.driver.mapping.annotations.Query;

@Singleton
@DatasourceLayer(type=DatasourceType.CASSANDRA , env=DatasourceEnvironment.PRODUCTION)
public class EmployeeTimeConfigCassandraManager 
	extends Cassandra3Manager
	implements EmployeeTimeConfigManager{

	
	private Accessors accessors;
	private Function<Row , Optional<EmployeeTimeConfig>> fnConvertToEntity = r -> convertRowToEntity(r);

	@Override
	protected Class<?> getInterfaceDefineColumns() {
		return F.class;
	}

	@Override
	protected String[] getPartitionKey() {
		return new String[]{ F.COMPANYID , F.MEMBERID };
	}

	@Override
	protected String[] getClusteringKey() {
		return null;
	}

	@Override
	protected void constructor() {
		this.accessors = this.createAccessors(Accessors.class);
	}

	@Override
	protected String getTableName() {
		return Accessors.TABLE_NAME;
	}

	private interface F extends PatternSqlBase{
		@Column
		public final String COMPANYID = "COMPANYID";
		@Column
		public final String MEMBERID = "MEMBERID";
		@Column
		public final String WORKSHIFT = "WORKSHIFT";
		
		
		public final String PATTERN_COLUMN = "(" 
				+ COMPANYID + "," + MEMBERID  + "," + WORKSHIFT
				+ ")";
		public final String PATTERN_COLUMN_VALUE = " VALUES("+ "?,?,?"
				+ ")";
	}
	@Accessor
	private interface Accessors extends F{
		public final String TABLE_NAME = "EMPLOYEE_TIME_CONFIG";
		public final String QUERY_TABLE_WHERE = PATTERN_SELECT + TABLE_NAME + PATTERN_WHERE;
		
		@Query(QUERY_TABLE_WHERE + COMPANYID + " = ? AND " + MEMBERID + " = ?")
		public ResultSet searchByKey(String comId , String memberId);
		
		@Query(PATTERN_INSERT + TABLE_NAME + PATTERN_COLUMN + PATTERN_COLUMN_VALUE + PATTERN_NOT_EXISTS)
		public ResultSet insert(String companyId,String memberId , String worktime);
		
		@Query(PATTERN_INSERT + TABLE_NAME + PATTERN_COLUMN + PATTERN_COLUMN_VALUE)
		public ResultSet save(String companyId,String memberId , String worktime);
	}
	
	private class EmployeeTimeConfigEntity extends EmployeeTimeConfigAbstract{
		private String companyId;
		private String memberId;
		private EmployeeTimeConfigEntity(){}
		public String getCompanyId() {	return companyId;	}
		public void setCompanyId(String companyId) {	this.companyId = companyId;	}
		public String getMemberId() {	return memberId;	}
		public void setMemberId(String memberId) {	this.memberId = memberId;	}
		
	}
	private EmployeeTimeConfigEntity createEntity(){
		return new EmployeeTimeConfigEntity();
	}
	
	private Optional<EmployeeTimeConfig> convertRowToEntity(Row r){
		try{
			EmployeeTimeConfigEntity e = this.createEntity();
			e.setCompanyId(r.getString(F.COMPANYID));
			e.setMemberId(r.getString(F.MEMBERID));
			e.setWorkShift(r.getString(F.WORKSHIFT));
			return Optional.of(e);
		}catch(Exception e){
			MyLog.error(this , e);
		}
		return Optional.empty();
	}
	
	public EmployeeTimeConfig createInstance(String companyId, String memberId){
		EmployeeTimeConfigEntity e = this.createEntity();
		e.setCompanyId(companyId);
		e.setMemberId(memberId);
		return e;
	}
	
	public StatusMessage save(EmployeeTimeConfig etc){
		try{
			this.accessors.save(etc.getCompanyId(), etc.getMemberId(), etc.getWorkShift());
			return StatusMessage.createInstanceSuccess();
		}catch(Exception e){
			MyLog.error(this ,e );
			return StatusMessage.createInstanceFail(e);
		}
	}
	
	public EmployeeTimeConfig getEmployeeTimeConfig(String companyId , String memberId){
		try{
			if(CSCCheckUtility.isBlankOrNull(companyId, memberId)) return null;
			
			return fetchOneResultSetToEntity(this.accessors.searchByKey(companyId, memberId), this.fnConvertToEntity);
		}catch(Exception e){
			MyLog.error(this , e);
		}
		return null;
	}
}
