package com.csc.library.zeesecurity.datasource.cassandra.manager;

import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.function.Function;

import com.csc.library.cassandra.Cassandra3Manager;
import com.csc.library.cassandra.annotation.Column;
import com.csc.library.cassandra.util.PatternSqlBase;
import com.csc.library.status.StatusMessage;
import com.csc.library.system.layer.DatasourceEnvironment;
import com.csc.library.system.layer.DatasourceLayer;
import com.csc.library.system.layer.DatasourceType;
import com.csc.library.utilities.CSCCheckUtility;
import com.csc.library.utilities.MyLog;
import com.csc.library.zeesecurity.databean.entity.WorkShift;
import com.csc.library.zeesecurity.databean.entity.impl.WorkShiftAbstract;
import com.csc.library.zeesecurity.databean.manager.WorkShiftManager;
import com.datastax.driver.core.ResultSet;
import com.datastax.driver.core.Row;
import com.datastax.driver.mapping.annotations.Accessor;
import com.datastax.driver.mapping.annotations.Query;


@DatasourceLayer(type=DatasourceType.CASSANDRA , env=DatasourceEnvironment.PRODUCTION)
public class WorkshiftCassandraManager extends Cassandra3Manager implements WorkShiftManager {

	private Accessors accessors;
	private Function<Row , Optional<WorkShift>> fnConvertToEntity = r -> convertRowToEntity(r);

	@Override
	protected Class<?> getInterfaceDefineColumns() {
		return F.class;
	}

	@Override
	protected String[] getPartitionKey() {
		return new String[]{ F.COMPANYID };
	}

	@Override
	protected String[] getClusteringKey() {
		return new String[]{ F.WORKSHIFTID };
	}

	@Override
	protected void constructor() {
		this.accessors = this.createAccessors(Accessors.class);
	}

	@Override
	protected String getTableName() {
		return Accessors.TABLE_NAME;
	}

	private interface F extends PatternSqlBase{
		@Column
		public final String COMPANYID = "COMPANYID";
		@Column
		public final String WORKSHIFTID = "WORKSHIFTID";
		@Column
		public final String DESCRIPTION = "DESCRIPTION";
		
		public final String PATTERN_COLUMN = "(" 
				+ COMPANYID + "," + WORKSHIFTID  + "," + DESCRIPTION
				+ ")";
		public final String PATTERN_COLUMN_VALUE = " VALUES("+ "?,?,?"
				+ ")";
	}
	@Accessor
	private interface Accessors extends F{
		public final String TABLE_NAME = "WORKSHIFT";
		public final String QUERY_TABLE_WHERE = PATTERN_SELECT + TABLE_NAME + PATTERN_WHERE;
		
		@Query(QUERY_TABLE_WHERE + COMPANYID + " = ? AND " + WORKSHIFTID + " = ?")
		public ResultSet getWorkshit(String comId , String workshiftId);
		
		@Query(QUERY_TABLE_WHERE + COMPANYID + " = ?")
		public ResultSet searchByCompanyId(String comId);
		
		@Query(PATTERN_INSERT + TABLE_NAME + PATTERN_COLUMN + PATTERN_COLUMN_VALUE + PATTERN_NOT_EXISTS)
		public ResultSet insert(String companyId,String workshiftId , String description);
		
		@Query(PATTERN_INSERT + TABLE_NAME + PATTERN_COLUMN + PATTERN_COLUMN_VALUE)
		public ResultSet save(String companyId,String workshiftId , String description);
	}
	
	private class WorkShiftEntity extends WorkShiftAbstract{
		private String companyId;
		private String workshiftId;
		private WorkShiftEntity(){}
		public String getCompanyId() {	return companyId;	}
		public void setCompanyId(String companyId) {	this.companyId = companyId;	}
		public String getWorkshiftId() {	return workshiftId;	}
		public void setWorkshiftId(String workshiftId) {	this.workshiftId = workshiftId;	}
		
	}
	private WorkShiftEntity createEntity(){
		return new WorkShiftEntity();
	}
	
	private Optional<WorkShift> convertRowToEntity(Row r){
		try{
			WorkShiftEntity ws = this.createEntity();
			ws.setCompanyId(r.getString(F.COMPANYID));
			ws.setWorkshiftId(r.getString(F.WORKSHIFTID));
			ws.setDescription(r.getString(F.DESCRIPTION));
			return Optional.of(ws);
		}catch(Exception e){
			MyLog.error(this , e);
		}
		return Optional.empty();
	}
	
	public WorkShift createInstance(String companyId){
		WorkShiftEntity e = this.createEntity();
		e.setCompanyId(companyId);
		e.setWorkshiftId(this.genUuid());
		return e;
	}
	
	public StatusMessage save(WorkShift ws){
		try{
			this.accessors.save(ws.getCompanyId(), ws.getWorkshiftId(), ws.getDescription());
			return StatusMessage.createInstanceSuccess();
		}catch(Exception e){
			MyLog.error(this ,e );
			return StatusMessage.createInstanceFail(e);
		}
	}
	
	public List<WorkShift> findWorkShiftByCompanyid(String companyId){
		try{
			if(CSCCheckUtility.isBlankOrNull(companyId)) return Collections.emptyList();
			
			return fetchResultSet(this.accessors.searchByCompanyId(companyId), this.fnConvertToEntity);
		}catch(Exception e){
			MyLog.error(this , e);
		}
		return Collections.emptyList();
	}
}
