package com.csc.library.zeesecurity.api;

import java.util.List;
import java.util.stream.Collectors;

import com.csc.library.cdi.GetCdiInstance;
import com.csc.library.command.CommandObject;
import com.csc.library.request.CommandState;
import com.csc.library.utilities.CSCCheckUtility;
import com.csc.library.utilities.CscCalendar;
import com.csc.library.utilities.CscCalendarSystem;
import com.csc.library.utilities.MyLog;
import com.csc.library.zeesecurity.databean.entity.CaptureTimestamp;
import com.csc.library.zeesecurity.databean.manager.CaptureTimestampManager;
import com.csc.library.zeesecurity.model.CaptureTimestampModel;
import com.csc.library.zeesecurity.request.CompanyRequestModel;
import com.csc.library.zeesecurity.response.CaptureTimestampResponseModel;

public class GetCaptureTimestampApi extends CommandObject<CompanyRequestModel>{

	@Override
	public void execute(CompanyRequestModel param) throws Exception {
		this.sendMessage("Processing GetListEmployee Api");
		CaptureTimestampResponseModel response = new CaptureTimestampResponseModel();
		response.setSuccess(false);
		this.setResultJson(response);
		try{
			CaptureTimestampManager ctManager = GetCdiInstance.of(CaptureTimestampManager.class);
			List<CaptureTimestamp> listCt = null;
			if(CSCCheckUtility.isBlankOrNull(param.getDateId())){
				listCt = ctManager.findCaptureTimestampByCompany(param.getCompanyId());
			}else{
				CscCalendar startDate = new CscCalendar(CscCalendarSystem.getFullDatetimeFromYYYYMMDD(param.getDateId()));
				CscCalendar endDate = new CscCalendar(startDate.getTimeInMillis());
				endDate.incDate();
				endDate.decMinute();
				listCt = ctManager.findCaptureTimestampByCompanyAndBetweenDatetime(param.getCompanyId() , startDate , endDate);
			}
			ctManager.findCaptureTimestampByCompany(param.getCompanyId());
			List<CaptureTimestampModel> listModel = listCt.stream()
				.map(CaptureTimestampModel::fromEntity)
				.collect(Collectors.toList());
			response.setList(listModel);
			response.setSuccess(true);
			this.setCommandState(CommandState.success);
		}catch(Exception e){
			MyLog.error(this, e);
			this.setCommandState(CommandState.fail);
		}
	}

}
