package com.csc.library.zeesecurity.api;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import com.csc.library.cdi.GetCdiInstance;
import com.csc.library.command.CommandObject;
import com.csc.library.download.HandleEmployeeDownloadUtil;
import com.csc.library.entity.CompanyEmployeeProfile;
import com.csc.library.entity.TimetempDataCom;
import com.csc.library.entity.TimetempDataCom.SwipeType;
import com.csc.library.entity.manager.CompanyEmployeeManager.EmployeeWorkingManager;
import com.csc.library.entity.manager.TimetempDataComManager;
import com.csc.library.request.CommandState;
import com.csc.library.utilities.CSCCheckUtility;
import com.csc.library.utilities.CscCalendar;
import com.csc.library.utilities.CscCalendarSystem;
import com.csc.library.utilities.MyLog;
import com.csc.library.zeesecurity.model.DailyPatrolCheckModel;
import com.csc.library.zeesecurity.request.DailyPetrolCheckRequestModel;
import com.csc.library.zeesecurity.response.DailyPetrolCheckResponseModel;

public class GetDailyPetrolCheckApi extends CommandObject<DailyPetrolCheckRequestModel> {

	@Override
	public void execute(DailyPetrolCheckRequestModel param) throws Exception {
		this.sendMessage("Processing GetDailyPetrolCheck Api");
		DailyPetrolCheckResponseModel response = new DailyPetrolCheckResponseModel();
		this.setResultJson(response);
		response.setSuccess(false);
		response.setDplist(Collections.emptyList());
		try{
			if(CSCCheckUtility.isBlankOrNull(param.getCompanyId(), param.getBranchId() , param.getDateId())){
				response.setMessage("Data Require is Empty.");
				response.setDplist(Collections.emptyList());
				this.setCommandState(CommandState.success);
				return;
			}
			EmployeeWorkingManager.View viewEmp = GetCdiInstance.of(EmployeeWorkingManager.class).getView();
			List<CompanyEmployeeProfile> listEmp = viewEmp.findEmployeeProfileByBranch(param.getBranchId());
			
			TimetempDataComManager ttManager = GetCdiInstance.of(TimetempDataComManager.class);
			CscCalendar startDate = new CscCalendar(CscCalendarSystem.getFullDatetimeFromYYYYMMDD(param.getDateId()));
			CscCalendar endDate = new CscCalendar(startDate.getTimeInMillis());
			endDate.incDate();
			if(listEmp.size() > 0){
				List<DailyPatrolCheckModel> listResult = new ArrayList<DailyPatrolCheckModel>(listEmp.size());
				response.setDplist(listResult);
				HandleEmployeeDownloadUtil handle = GetCdiInstance.of(HandleEmployeeDownloadUtil.class);
				for(CompanyEmployeeProfile e : listEmp){
					try{
						List<TimetempDataCom> listTimetemp = ttManager.findTimetempDateBetween(e.getCompanyID(), e.getMemberID()
								, startDate , endDate);
						DailyPatrolCheckModel model = new DailyPatrolCheckModel();
						listResult.add(model);
						model.setCompanyId(e.getCompanyID());
						model.setMemberId(e.getMemberID());
						model.setEmployeeCode(e.getEmployeeCode());
						model.setMemberName(e.getMemberName());
						model.setLagacyCode(e.getLagacyCode());
						model.setBranchId(e.getBranchID());
						model.setBranchName(e.getBranchTname());
						model.setPosition(e.getPosition());
						model.setDepartment(e.getDepartment());
						model.setFileName(handle.getDownloadUrlPictureMember(e));
						
						this.setTimeFlagbyAnyTimeInDay(model, listTimetemp, startDate);
					}catch(Exception ex){}
				}
			}
			this.setCommandState(CommandState.success);
		}catch(Exception e){
			MyLog.error(this , e);
			this.setCommandState(CommandState.fail);
		}
	}
	
	private void setTimeFlagbyAnyTimeInDay(DailyPatrolCheckModel model , List<TimetempDataCom> listTimetemp , CscCalendar dateId){
		try{
			int[] tmp = new int[3];
			for(TimetempDataCom tt : listTimetemp){
				try {
					CscCalendar c = new CscCalendar(tt.getSwipeDatetime());
					if(!dateId.equalsDate(c)){
						continue;
					}
					if(tt.getSwipeType() == SwipeType.In){
						tmp[0] = 1;
					}else if(tt.getSwipeType() == SwipeType.BreakIn || tt.getSwipeType() == SwipeType.BreakOut){
						tmp[1] = 1;
					}else if(tt.getSwipeType() == SwipeType.Out){
						tmp[2] = 1;
					}
					if(tmp[0] == 1 && tmp[1] == 1 && tmp[2] == 1){
						break;
					}
				} catch (Exception e) {}
			}
			model.setTimeinCheck(tmp[0]);
			model.setSurveyCheck(tmp[1]);
			model.setTimeoutCheck(tmp[2]);
		}catch(Exception ex){}
	}
	
	private void setTimeFlagByDefault(DailyPatrolCheckModel model , List<TimetempDataCom> listTimetemp , CscCalendar dateId){
		try{
			CscCalendar[] tmp = new CscCalendar[3];
			List<TimetempDataCom> listTimetempSort = listTimetemp.stream().sorted(new Comparator<TimetempDataCom>() {
				@Override
				public int compare(TimetempDataCom o1, TimetempDataCom o2) {
					try {
						return CscCalendarSystem.parse(o1.getSwipeDatetime()).getTime() > CscCalendarSystem.parse(o2.getSwipeDatetime()).getTime() ? 1 : -1;
					} catch (ParseException e) {
						e.printStackTrace();
					}
					return 0;
				}
			}).collect(Collectors.toList());
			
			for(TimetempDataCom tt : listTimetempSort){
				try{
					CscCalendar c = new CscCalendar(tt.getSwipeDatetime());
					if(tmp[0] == null){
						if(c.equalsDate(dateId)){
							if(tt.getSwipeType() == SwipeType.In){
								tmp[0] = c;
								continue;
							}
						}else{
							//ไม่ต้องทำอะไรต่อ เพราะเป็นเข้าของอีกวันไปแล้ว
							break;
						}
					}else if(tmp[1] == null){
						if(!c.equalsDate(dateId) && tt.getSwipeType() == SwipeType.In){
							break;
						}
						if(tt.getSwipeType() == SwipeType.Out){
							tmp[2] = c;
							break;
						}
						if(tt.getSwipeType() == SwipeType.BreakIn || tt.getSwipeType() == SwipeType.BreakOut){
							tmp[1] = c;
							continue;
						}
					}else{
						if(!c.equalsDate(dateId) && tt.getSwipeType() == SwipeType.In){
							break;
						}
						if(tt.getSwipeType() == SwipeType.Out){
							tmp[2] = c;
							break;
						}
					}
				}catch(Exception exx){}
			}
			model.setTimeinCheck(tmp[0] != null ? 1 : 0);
			model.setSurveyCheck(tmp[1] != null ? 1 : 0);
			model.setTimeoutCheck(tmp[2] != null ? 1 : 0);
		}catch(Exception ex){}
	}

}
