package com.csc.library.zeesecurity.api;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import com.csc.library.cdi.GetCdiInstance;
import com.csc.library.command.CommandObject;
import com.csc.library.entity.CompanyEmployeeProfile;
import com.csc.library.entity.manager.CompanyEmployeeManager.EmployeeWorkingManager;
import com.csc.library.request.CommandState;
import com.csc.library.utilities.CSCCheckUtility;
import com.csc.library.utilities.MyLog;
import com.csc.library.zeesecurity.databean.entity.EmployeeReplacement;
import com.csc.library.zeesecurity.databean.manager.EmployeeReplacementManager;
import com.csc.library.zeesecurity.model.EmployeeModel;
import com.csc.library.zeesecurity.model.ReplacementModel;
import com.csc.library.zeesecurity.request.ListEmployeeRequestModel;
import com.csc.library.zeesecurity.response.EmployeeListResponseModel;

public class GetListEmployeeApi extends CommandObject<ListEmployeeRequestModel>{

	@Override
	public void execute(ListEmployeeRequestModel param) throws Exception {
		this.sendMessage("Processing GetListEmployee Api");
		EmployeeListResponseModel response = new EmployeeListResponseModel();
		this.setResultJson(response);
		try{
			if(CSCCheckUtility.isBlankOrNull(param.getCompanyId())){
				response.setEmployeeList(Collections.emptyList());
				response.setReplacementList(Collections.emptyList());
				this.setCommandState(CommandState.success);
				return;
			}
			EmployeeWorkingManager.View viewEmp = GetCdiInstance.of(EmployeeWorkingManager.class).getView();
			List<CompanyEmployeeProfile> listEmp = viewEmp.findEmployeeProfileByCompanyId(param.getCompanyId());
			List<EmployeeModel> listEmpModel = listEmp.stream()
				.map(EmployeeModel::fromEntity)
				.collect(Collectors.toList());
			response.setEmployeeList(listEmpModel);
			
			EmployeeReplacementManager erManager = GetCdiInstance.of(EmployeeReplacementManager.class);
			
			List<EmployeeReplacement> listEr = erManager.findEmployeeReplacementByCompany(param.getCompanyId());
			List<ReplacementModel> listReplaceModel = listEr.stream()
					.map(ReplacementModel::fromEntity)
					.collect(Collectors.toList());
			response.setReplacementList(listReplaceModel);
			this.setCommandState(CommandState.success);
		}catch(Exception e){
			MyLog.error(this, e);
			this.setCommandState(CommandState.fail);
		}
	}

}
