package com.csc.library.zeesecurity.api;

import com.csc.library.cdi.GetCdiInstance;
import com.csc.library.command.CommandObject;
import com.csc.library.request.CommandState;
import com.csc.library.status.StatusMessage;
import com.csc.library.utilities.CSCCheckUtility;
import com.csc.library.utilities.CscCalendarSystem;
import com.csc.library.utilities.MyLog;
import com.csc.library.zeesecurity.databean.entity.EmployeeReplacement;
import com.csc.library.zeesecurity.databean.manager.EmployeeReplacementManager;
import com.csc.library.zeesecurity.request.SaveReplacementRequestModel;
import com.csc.library.zeesecurity.response.StatusResponseModel;

public class SaveReplacementApi extends CommandObject<SaveReplacementRequestModel>{

	@Override
	public void execute(SaveReplacementRequestModel param) throws Exception {
		this.sendMessage("Processing SaveReplacement Api");
		StatusResponseModel responseModel = new StatusResponseModel();
		responseModel.setSuccess(false);
		this.setResultJson(responseModel);
		try {
			if(CSCCheckUtility.isBlankOrNull(param.getCompanyId() , param.getMemberId() , param.getSubMemberId()
					, param.getStartDate() , param.getEndDate())){
				responseModel.setMessage("Data Require is Empty.");				
				this.setCommandState(CommandState.success);
				return;
			}
			EmployeeReplacementManager rep = GetCdiInstance.of(EmployeeReplacementManager.class);
			EmployeeReplacement e = rep.createInstance(param.getCompanyId()	, param.getMemberId());
			e.setMemberIdReplacement(param.getSubMemberId());
			e.setStartDate(CscCalendarSystem.getFullDatetimeFromYYYYMMDD(param.getStartDate()));
			e.setEndDate(CscCalendarSystem.getFullDatetimeFromYYYYMMDD(param.getEndDate()));
			StatusMessage sm = rep.save(e);
			if(sm.isSuccess()){
				responseModel.setSuccess(true);
			}else{
				responseModel.setSuccess(false);
				responseModel.setMessage(sm.getMessage());
			}
			this.setCommandState(CommandState.success);
		} catch (Exception e) {
			MyLog.error(this , e);
			this.setCommandState(CommandState.fail);
		}
	}

}
